<?php

namespace App\Http\Controllers;

use App\Models\Goal;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;

class GoalController extends Controller
{

    public function __construct()
    {
        $this->middleware('permission:ver-meta|crear-meta|editar-meta|borrar-meta', ['only' => ['index', 'show']]);
        $this->middleware('permission:crear-meta', ['only' => ['create', 'store']]);
        $this->middleware('permission:editar-meta', ['only' => ['edit', 'update']]);
        $this->middleware('permission:borrar-meta', ['only' => ['destroy']]);
    }


    public function index()
    {
        // Obtener el número de la semana actual
        $currentWeekNumber = Carbon::now()->weekOfYear;

        $userId = auth()->id();
        // Obtener solo los objetivos de la semana actual
        $goals = Goal::where('week_number', $currentWeekNumber)->where('user_id', $userId)->paginate(10);

        return view('goals.index', compact('goals', 'currentWeekNumber'));
    }


    public function progress()
    {
        $currentWeekNumber = Carbon::now()->weekOfYear;
        $userId = auth()->id(); // Filtrar por usuario autenticado

        // Obtener metas de la semana actual
        $weeklyGoals = Goal::where('week_number', $currentWeekNumber)
            ->where('user_id', $userId)
            ->get();

        // Calcular el total de metas y completadas
        $totalGoals = $weeklyGoals->sum('total_goals');
        $completedGoals = $weeklyGoals->sum('completed_goals');

        // Calcular el porcentaje de progreso
        $progress = ($totalGoals > 0) ? ($completedGoals / $totalGoals) * 100 : 0;

        return view('goals.progress', compact('weeklyGoals', 'currentWeekNumber', 'totalGoals', 'completedGoals', 'progress'));
    }

    public function updateProgress(Request $request, Goal $goal)
    {
        $change = $request->input('change');

        // Asegurar que el número de metas completadas no exceda el total ni sea negativo
        $goal->completed_goals = max(0, min($goal->completed_goals + $change, $goal->total_goals));

        if ($goal->total_goals > 0) {
            $goal->progress = ($goal->completed_goals / $goal->total_goals) * 100;
        } else {
            $goal->progress = 0; // Evita errores si total_goals es 0
        }

        $goal->save();

        return response()->json(['success' => true, 'completed_goals' => $goal->completed_goals]);
    }


    public function create()
    {
        return view('goals.create');
    }


    public function store(Request $request)
    {
        $validated = $request->validate([
            'title' => [
                'required',
                'string',
                'max:50',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'description' => [
                'nullable',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'total_goals' => 'required|integer|min:1',
            'completed_goals' => 'required|integer|min:0',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
        ]);

        // Obtener el número de la semana actual
        $weekNumber = Carbon::now()->weekOfYear;

        // Obtener la fecha del lunes de la semana actual
        $weekStartDate = Carbon::now()->startOfWeek()->toDateString();

        $goal = new Goal();
        $goal->title = $request->title;
        $goal->description = $request->description;
        $goal->total_goals = $request->total_goals;
        $goal->completed_goals = $request->completed_goals;
        $goal->start_date = $request->start_date;
        $goal->end_date = $request->end_date;
        $goal->user_id = auth()->id();

        // Asignar automáticamente el número de semana y la fecha de inicio de la semana
        $goal->week_number = $weekNumber;
        $goal->week_start_date = $weekStartDate;

        $goal->save();

        return redirect()->route('goals.index')->with('success', 'Meta creado exitosamente.');
    }


    public function show(Request $request)
    {

        $weekNumber = $request->input('week_number', Carbon::now()->weekOfYear);

        $goals = Goal::where('user_id', auth()->id())
            ->where('week_number', '=', $weekNumber)
            ->orderBy('week_number', 'desc')
            ->get();

        // Calcular el porcentaje de progreso por meta
        $goals->each(function ($goal) {
            $goal->percentage = $goal->total_goals > 0 ? ($goal->completed_goals / $goal->total_goals) * 100 : 0;
        });

        return view('goals.previous', [
            'goals' => $goals,
            'currentWeekNumber' => $weekNumber
        ]);
    }


    public function edit(Goal $goal)
    {
        return view('goals.edit', compact('goal'));
    }


    public function update(Request $request, Goal $goal)
    {
        $validated = $request->validate([
            'title' => [
                'required',
                'string',
                'max:50',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'description' => [
                'nullable',
                'string',
                'max:255',
                'regex:/^[a-zA-Z0-9áéíóúÁÉÍÓÚüÜñÑ\s\.\-\"\'\,]+$/',
            ],
            'total_goals' => 'required|integer|min:1',
            'completed_goals' => 'required|integer|min:0',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
        ]);

        $goal->title = $request->title;
        $goal->description = $request->description;
        $goal->total_goals = $request->total_goals;
        $goal->completed_goals = $request->completed_goals;
        $goal->start_date = $request->start_date;
        $goal->end_date = $request->end_date;

        $goal->save();

        return redirect()->route('goals.index')->with('success', 'Meta actualizado exitosamente.');
    }


    public function destroy(Goal $goal)
    {
        $goal->delete();

        return redirect()->route('goals.index')->with('deleted', 'Meta eliminado exitosamente.');
    }
}
